#!/usr/bin/env python

import os,sys,argparse,serial
import struct
from time import sleep

from senao_codes import OPCODES, senao_hci_read

def init_serial(port):
    bt = serial.Serial()
    if os.name == 'posix':
        if port is not None:
            bt.port = port
        else:
            bt.port = "/dev/ttyACM0"
    else:
        bt.port = "COM3"
    #bt.baudrate = 57600
    bt.baudrate = 115200
    bt.stopbits = 1
    bt.parity = 'N'
    bt.bytesize = 8
    bt.xonxoff = 0
    bt.rtscts = 0
    bt.timeout = None
    try:
        bt.open()
    except:
        print bt.port+" cannot be open."
        sys.exit("Senao HCI Terminated.")

    print bt.port+" is open."
    return bt

def auto_int(x):
    return int(x, 0)

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Senao HCI usage')
    parser.add_argument("-s","--serial", metavar='port', help="assign prot to do HCI, ex:/dev/ttyACM0")
    parser.add_argument("-o","--opcode", type=auto_int, metavar='opcode', required=True, help="do specific operation, ex:0xFE00")
    parser.add_argument("-r","--raw", type=auto_int, nargs='*', metavar='rawdata', help="raw data to write.")

    args = parser.parse_args()
    bt = init_serial(args.serial)
    print "Senao HCI: Operation "+str(hex(args.opcode))+" is executing."

    #Sending command to client
    try:
        OPCODES[hex(args.opcode)](bt, args.raw)
    except:
        print str(hex(args.opcode))+" is not supported."
        sys.exit("Senao HCI Terminated.")

    #Get response from client
    if bt.isOpen():
        #wait a while to get response
        sleep(0.05)
        try:
            senao_hci_read(bt)
        except:
            sys.exit("Senao HCI Terminated.")

    sys.exit("Senao HCI Finished.")
